#!/bin/sh -xv

# Exit script immediately on any errors
set -e

# Function to check and normalize modified LFS files
normalize_lfs_files() {
    # Flag to indicate whether any LFS files were renormalized
    renormalized_files=false
    
    # Get the list of modified files
    modified_files=$(git status --porcelain | awk '{if ($1 == "M") print $2}')
    
    # Loop over each modified file to check if it is managed by LFS
    for file in $modified_files; do
        is_lfs_file=$(git check-attr filter $file | grep "filter: lfs")
        
        # If the file is managed by LFS, then renormalize it
        if [ ! -z "$is_lfs_file" ]; then
            echo "git add $file --renormalize"
            git add $file --renormalize
            
            # Mark that LFS files have been renormalized
            renormalized_files=true
        fi
    done
    
    # If any LFS files were renormalized, commit the changes
    if $renormalized_files ; then
        echo "git commit -m 'Renormalized LFS files'"
        git commit -m "Renormalized LFS files"
    fi
}

# Fetch from origin remote (github)
#echo "git fetch --force origin"
git fetch --force origin

# set git data necessary to commit
git config --replace-all user.email $GITLAB_USER_EMAIL
git config --replace-all user.name "$GITLAB_USER_NAME"
git config checkout.defaultRemote origin

# Get the list of all branches on origin
branches=$(git branch -r | grep "origin/" | sed 's/origin\///' | grep -v HEAD)

# Loop over all branches from origin
for branch in $branches; do

    echo "************************************************************"
    echo "Checking branch $branch"
    echo "************************************************************"

    # Check if the github branch exists locally
    if git rev-parse --verify "$branch" > /dev/null 2>&1; then
        echo "Branch $branch already exists locally."
    else
        # Checkout  branch locally with --force option to create it
        echo "git checkout --force $branch"
        git checkout --force $branch
    fi

    # Check if the github branch is ahead of the local branch, pull it
    behind=$(git rev-list --count $branch..origin/$branch)
 
    if [ $behind -eq 0 ]; then
        # If the wvsrepo branch exists and is at the same commit, then we can skip this branch
        if git rev-parse --verify "wvsrepo/$branch" > /dev/null 2>&1; then
            echo "wvsrepo $branch already exists locally."
            ahead=$(git rev-list --count wvsrepo/$branch..$branch)

            if [ $ahead -eq 0 ]; then
                echo "Local wvsrepo $branch is not ahead of remote. Skipping."
                continue
            fi
        fi
    fi

    # Fetch all LFS objects for the current branch
    git checkout --force $branch
    git pull --no-ff --log=250

    # Fetch all LFS objects for the current branch
    echo "git lfs fetch --all"
    git lfs fetch --all
    
    # Normalize LFS files if there are any modifications
    #normalize_lfs_files
    
    # Push all fetched LFS objects to the wvsrepo remote
    echo "git lfs push wvsrepo --all"
    git lfs push wvsrepo --all
    
    # Push any changes (including LFS pointer changes) for the current branch to the wvsrepo remote
    echo "git push --force wvsrepo $branch"
    git push --force wvsrepo $branch
done

# Print completion message
echo "LFS objects from all branches on 'origin' have been fetched, renormalized, and pushed to 'wvsrepo'."
